#include "ds1620.h"

unsigned char ds1620_init (void)
{
	/*set all pins as output*/
	DDR_DS1620 = (1<<DQ_DS1620) | (1<<CLK_DS1620) | (1<<RES_DS1620);
	/*set CPU use bit, continuously temperature conversion*/
	ds1620_write_config (0x02);
	/*wait, while EEPROM busy --> check NVB bit*/
	while (ds1620_read_config () & (1<<4));
	/*start continuous conversion*/
	PORT_DS1620 |= (1<<RES_DS1620);
	ds1620_put_byte (0xEE);
	PORT_DS1620 &= ~(1<<RES_DS1620);
	//while (!(ds1620_read_config () & (1<<7)));
	/*return configuration, if returned configuration == 0x02
	everything is alright*/
	return ds1620_read_config ();
}


void ds1620_write_config (unsigned char byte)
{
	/*set RES high*/
	PORT_DS1620 |= (1<<RES_DS1620);
	/*send write config command*/
	ds1620_put_byte (0x0C);
	/*sends user-definable value, which will be written in config-register*/
	ds1620_put_byte (byte);
	/*ground RES*/
	PORT_DS1620 &= ~(1<<RES_DS1620);
}


unsigned char ds1620_read_config (void)
{
	PORT_DS1620 |= (1<<RES_DS1620);
	/*send read config command*/
	ds1620_put_byte (0xAC);
	/*receive value stored in config-register*/
	unsigned char byte = ds1620_get_byte ();
	PORT_DS1620 &= ~(1<<RES_DS1620);
	return byte;
}


unsigned char ds1620_get_temp (void)
{
	PORT_DS1620 |= (1<<RES_DS1620);
	/*send read temperature command*/
	ds1620_put_byte (0xAA);
	/*receive values stored in temperature-registers*/
	unsigned char byte = ds1620_get_byte ();
	PORT_DS1620 &= ~(1<<RES_DS1620);
	return byte;
}


void ds1620_put_byte (unsigned char byte)
{
	PORT_DS1620 |= (1<<DQ_DS1620);
	for (unsigned char i=0; i<8; i++)
	{
		PORT_DS1620 &= ~(1<<CLK_DS1620);
		if (byte & 0x01)
		{
			PORT_DS1620 |= (1<<DQ_DS1620);
		}
		else
		{
			PORT_DS1620 &= ~(1<<DQ_DS1620);
		}
		PORT_DS1620 |= (1<<CLK_DS1620);
		byte = (byte >> 1);
	}
	PORT_DS1620 &= ~(1<<DQ_DS1620);
}


unsigned char ds1620_get_byte (void)
{
	unsigned char byte = 0;
    DDR_DS1620 &= ~(1<<DQ_DS1620);
	for (unsigned char i=0; i<8; i++)
	{
		PORT_DS1620 &= ~(1<<CLK_DS1620);
		if (PIN_DS1620 & (1<<DQ_DS1620))
			byte |= 0x80;
		PORT_DS1620 |= (1<<CLK_DS1620);
		byte = (byte >> 1);
	}
	DDR_DS1620 |= (1<<DQ_DS1620);
	return byte;
}










